;; Author: Rory Sie
;; Email: rory.sie@gmail.com
;; ----------------
;; version: v0.1
;; Requirements: Netlogo 4.1.3
;; ----------------
;; GENERAL TODOs: 
;; - make idea tangible: vector of 5 characteristics?
;; - replace Belbin personality with Idea (vector) similarity
;; - for winning coalition: idea characteristics that score high (characteristic x idea value x power) should be implemented
;; - for winning idea: update power accordingly
;; - monitor innovation diffusion?

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; global variables
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
extensions [matrix]

directed-link-breed[interestedIns interestedIn]
undirected-link-breed[coalitions coalition]
undirected-link-breed[relationships relationship]

turtles-own [
  similarity-to-other-turtles
  skills ;;not used at the moment
  old-power 
  power
  power-average 
  idea-value-average
  belbin-role 
  rank ;;not used at the moment
  creativity 
  age 
  gender 
  idea-value
  agent-scores
  coalition-rates
  distance-from-other-turtles
  coalition-path-length
  number-of-coalitions
  number-of-wins
]

globals [
  counter 
  infinity
  total-payoff
  skill-list 
  rank-list 
  belbin-role-list
  CB ;; betweenness centrality for all agents
  dyad-scores
  agent-individual-gains
  shapley-values
  coalition-matrix
  coalition-list
  coalition-counter
  coalition-strengths
  total-number-of-coalitions
  winning-coalition
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;; reporters
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  average-power-per-winning-coalition
  average-betweenness-per-winning-coalition
  average-idea-value-per-winning-coalition
  
  max-idea-value-per-winning-coalition 
  max-betweenness-per-winning-coalition
  max-power-per-winning-coalition
  wins-list ;; number of wins per agent
  chance-of-winning-coalition
  winning-coalition-size
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; 
  ;; model weights
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  w1 ;; weight for betweenness centrality (organisation structure) to power
  w2 ;; weight for perceived value of an idea to power
  w3 ;; weight for betweenness centrality to creativity, afterwards multiplied by idea value
  w4 ;; weight for age (social identity) to power
  w5 ;; weight for social identity to actor similarity
  w6 ;; weight for power to coalitions
  w7 ;; weight for actor similarity to coalitions
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; 
  ;; other weights
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;    
  w8 ;; weight for belbin-similarity
  w9 ;; weight for gender similarity
]

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; setup functions
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

to setup
  clear-all ;; clear everything from the screen
  set counter 0
  set infinity 99999  ;; just an arbitrary choice for a large number
  
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;;set the weights for this simulation
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  set-weights
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;;setup lists for turtles
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  setup-skill-list
  setup-rank-list
  setup-belbin-role-list
  setup-coalition-matrix
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;;setup turtles
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  setup-turtles  
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;;setup pathes
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  setup-payoff
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;;setup links
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  setup-coalitions
  setup-relationships
  
  find-path-lengths ;; find the shortest path lengths for each agent
  calculate-betweenness-centrality ;; calculate betweenness centrality for each agent
  calculate-actor-similarity ;; calculate the similarity between agents
  set-creativity ;; compute creativity based on betweenness centrality
  set-wins-list ;; setup the number of wins per agent, which is zero for every agent at the start
  
  do-plots
end

to go
  if ticks >= 100 [ stop ] ;; stop after 100 ticks
  ;; calculate-betweenness-centrality ;; only needed if new relationships are formed
  ;; set-creativity ;; ;; only needed if new relationships are formed
  ;; move-turtles
  ask coalitions [ die]
  if hide-relationships = true [
    ask relationships [ hide-link]
  ]
  ask turtles [ set color green - 1]
  
  setup-coalition-matrix
  generate-ideas
  calculate-power
  normalize-power
  calculate-agent-scores
  calculate-dyads
  calculate-individual-gains
  calculate-shapley-values
  form-coalitions
  find-coalition-paths ;; find the coalition path length for each agent
  view-coalitions
  compute-coalition-strengths
  compute-winning-coalition
  compute-averages-per-winning-coalition ;;this is for reporting
  compute-average-power
  compute-average-idea-value
  compute-maximums-per-winning-coalition
  view-number-of-coalitions
  update-power
  tick ;; set tick + 1
  do-plots
end

to setup-patches
  ask patches [
    set pcolor white   
  ]
end

to setup-turtles
  create-ordered-turtles number-of-turtles [ fd 8 ]
  
  set-number-of-wins ;; every agent begins with zero wins
  
  
  while [counter < number-of-turtles] [
    ask turtle counter [ 
      set skills []
      set-age who
      set-gender who
      set-skills who
      set-rank who
      set-belbin-role who
      set-power who ;; set the power of the turtle
      set shape "turtle"
      set label who ;; set the label to turtle number. Can also be 'counter'
      set color green - 1
    ]
    set counter counter + 1 
  ]
  
  set-power-average ;; every agent's power average begins with initial power
  set-idea-value-average ;; every agent's idea value average begins with initial idea value
end

to setup-relationships
  ifelse network-type = "random2" [
    let turtlecount count turtles
    let total-links-possible turtlecount * (turtlecount - 1) / 2
    let number-of-links total-links-possible * network-density
    let iteration 0
    ;; 1 x number-of-turtles links are made, average number of links per agent is therefore 2
    while [iteration < number-of-links] [ 
      ask turtle random number-of-turtles [
        let randomAgent random number-of-turtles ;; select a random agent
        if randomAgent != who [  ;; if random agent is not equal to itself
          create-relationship-with turtle randomAgent [
            set color 47
            set thickness 0
          ];; create a relationship link to that random agent
        ]
      ]
      set iteration iteration + 1
    ]
  ][
  ifelse network-type = "random" [
    let iteration 0
    ;; 1 x number-of-turtles links are made, average number of links per agent is therefore 2
    while [iteration < 1 * number-of-turtles] [ 
      ask turtle random number-of-turtles [
        let randomAgent random number-of-turtles ;; select a random agent
        if randomAgent != who [  ;; if random agent is not equal to itself
          create-relationship-with turtle randomAgent [
            set color 47
            set thickness 0
          ];; create a relationship link to that random agent
        ]
      ]
      set iteration iteration + 1
    ]
  ][
    ifelse network-type = "regular lattice" [
      ask turtles [
        let neighbourAgent who + 1 ;; select a neighbouring agent
        if neighbourAgent = number-of-turtles [
          set neighbourAgent 0
        ]
        if neighbourAgent != who [  ;; if neighbouring agent is not equal to itself
          create-relationship-with turtle neighbourAgent [
            set color 47
            set thickness 0
          ];; create a relationship link to that neighbouring agent
        ]
      ]
      ask turtles [
        let neighbour2Agent who + 2 ;; select another neighbouring agent
        if neighbour2Agent = number-of-turtles [
          set neighbour2Agent 0
        ]
        if neighbour2Agent = number-of-turtles + 1 [
          set neighbour2Agent 1
        ]
        if neighbour2Agent != who [  ;; if neighbouring agent is not equal to itself
          create-relationship-with turtle neighbour2Agent [
            set color 47
            set thickness 0
          ];; create a relationship link to that neighbouring agent
        ]
      ]    
    ][
      ifelse network-type = "small world" [
        ask turtles [
          let neighbourAgent 0
          let randomness random number-of-turtles
          ifelse randomness < (number-of-turtles / 4) [
            set neighbourAgent random number-of-turtles
          ]
          [  
            set neighbourAgent who + 1 ;; select a neighbouring agent
          ]
          
          if neighbourAgent = number-of-turtles [
            set neighbourAgent 0
          ]
          if neighbourAgent != who [  ;; if neighbouring agent is not equal to itself
            create-relationship-with turtle neighbourAgent [
              set color 47
              set thickness 0
            ];; create a relationship link to that neighbouring agent
          ]
        ]
        ask turtles [
          let neighbour2Agent 0
          let randomness random number-of-turtles
          ifelse randomness < (number-of-turtles / 4) [
            set neighbour2Agent random number-of-turtles
          ]
          [  
            set neighbour2Agent who + 2 ;; select another neighbouring agent
          ]
          
          if neighbour2Agent = number-of-turtles [
            set neighbour2Agent 0
          ]
          if neighbour2Agent = number-of-turtles + 1 [
            set neighbour2Agent 1
          ]
          if neighbour2Agent != who [  ;; if neighbouring agent is not equal to itself
            create-relationship-with turtle neighbour2Agent [
              set color 47
              set thickness 0
            ];; create a relationship link to that neighbouring agent
          ]
        ]
      ][
        ;; TODO: insert preferential network
      ]  
    ]
  ]
  ]
end

to setup-coalitions
  ask coalitions [
    set color blue
    set thickness 3
  ]
end

to setup-payoff
  set total-payoff 10
end

to set-skills [number]
  foreach skill-list [   
    if random 3 = 1 [ ;; chances that a skill is assigned, is 0.33
      assign-skill-to-turtle number ? ;; assign current skill to turtle 'number'
    ]
  ]
end
 
to assign-skill-to-turtle [turtlenumber skillString]
  ask turtle turtlenumber [
    set skills lput skillString skills
    ;;print turtle counter print skills 
  ]
end

to set-age [turtlenumber]
  ask turtle turtlenumber [
    set age random 50 + 15
  ]
end

to set-gender [turtlenumber]
  ask turtle turtlenumber [
    set gender random 2 ;; 0 for female, 1 for male
  ]
end

to set-rank [turtlenumber]
  ask turtle turtlenumber [
    set rank item random 3 rank-list
  ]
end

to set-creativity
  ask turtles [
    set creativity w3 * item who CB
  ]
end

to set-belbin-role [turtlenumber]
  ask turtle turtlenumber [
    set belbin-role item random 9 belbin-role-list
  ]
end

to set-number-of-wins
  ask turtles [
    set number-of-wins 0
  ]
end

to set-wins-list
  let p 0
  set wins-list []
  while[p < number-of-turtles][
    set wins-list lput 0 wins-list  
    set p p + 1
  ]
end 

to set-power-average
  ask turtles [
    set power-average power
  ]
end

to set-idea-value-average
  ask turtles [
    set idea-value-average idea-value
  ]
end

to setup-rank-list
  set rank-list [1 2 3] ;; 1 is for associate, 2 is for manager, 3 is for director
end

to setup-skill-list
  set skill-list ["skill1" "skill2" "skill3"]
end

to setup-belbin-role-list
  set belbin-role-list ["Plant" "Monitor Evaluator" "Co-ordinator" "Resource Investigator" "Implementer" "Completer Finisher" "Teamworker" "Shaper" "Specialist"]
end

to setup-coalition-matrix
  let turtle-count count turtles
  set coalition-matrix matrix:make-constant turtle-count turtle-count -1
end

to set-power [turtlenumber]
  ask turtle turtlenumber [
    if ticks = 0 [
      set power random 100
    ]
  ]
end

to normalize-power
  let turtle-count count turtles
  let p 0
  let max-power 0
  while [p < turtle-count] [
    if [power] of turtle p > max-power [ ;; if agent's power is bigger than max-power
      set max-power [power] of turtle p ;; set new max-power
    ]
    set p p + 1
  ]
  ask turtles [
    set power power / max-power * 100 ;; normalize power
  ]
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; agent functions
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

to move-turtles ;; this is a procedure on how to move turtles and how much energy it costs to move
  ask turtles [
    right random 360
    forward 1
  ]
end

to generate-ideas
  ask turtles [
    set idea-value random 100 + creativity  
  ]
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; agent beliefs
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

to calculate-actor-similarity ;; calculate similarity in terms of age, gender and personality
  ask turtles [
    let turtle-count count turtles
    set similarity-to-other-turtles []
    let s 0
    while [s < turtle-count] [
      set similarity-to-other-turtles fput 0 similarity-to-other-turtles
      set s s + 1
    ]
    let p 0
    while [p < turtle-count] [
      let belbin-similarity 0
      let gender-similarity 0
      
      if belbin-role = [belbin-role] of turtle p [
        set belbin-similarity 100
      ]
      if gender = [gender] of turtle p [
        set gender-similarity 100
      ]
      
      let ageDifference 0
      if age - [age] of turtle p != 0 [
        set ageDifference (sqrt ((age - [age] of turtle p) * (age - [age] of turtle p))) 
        set ageDifference ageDifference * 2
        ;;type age type " " type [age] of turtle p type " " type "ageDifference:" print ageDifference
      ]
      let new-similarity (belbin-similarity * w8) + (gender-similarity * w9) - (ageDifference * w5)
      ;; calculate final similarity including weights
      set similarity-to-other-turtles replace-item p similarity-to-other-turtles new-similarity 
      
      set p p + 1
    ]
  ]
end

to calculate-agent-scores
  let turtle-count count turtles
  ask turtles [
    let p 0
    set agent-scores []
    
    while [p < turtle-count] [
      set agent-scores lput 0 agent-scores
      let agent-score 0
      let betweenness-centrality item p CB
      let power-of-agent [power] of turtle p
      let similarity-to-agent item p similarity-to-other-turtles
      ifelse betweenness-centrality != 0 [
        set agent-score similarity-to-agent * w7 + power-of-agent * w6 ;;+ w7 * betweenness-centrality + power-of-agent * w6
      ][
        set agent-score similarity-to-agent * w7 + power-of-agent * w6
      ]
      set agent-scores replace-item p agent-scores agent-score 
      
      set p p + 1
    ]  
  ]
end

to calculate-dyads
  let turtle-count count turtles
  set dyad-scores matrix:make-constant turtle-count turtle-count 0
  
  ask turtles [
    let p 0
    while [p < turtle-count] [
      if p != who [ ;; let coalition {who} be of zero value
        let new-value item p agent-scores + item who agent-scores 
        matrix:set dyad-scores who p new-value ;; let coalition {p,who} be of (super)additive value       
      ]
      set p p + 1
    ]
  ]
end

to calculate-individual-gains
  let turtle-count count turtles
  set agent-individual-gains []
  ask turtles [
    set agent-individual-gains lput 0 agent-individual-gains
  ]
end

to calculate-shapley-values
  let turtle-count count turtles
  set shapley-values matrix:make-constant turtle-count turtle-count 0
  ask turtles [
    let p 0
    while [p < turtle-count] [
      ;; shapley{i,j} = v{1,2} - v{1} - v{2}
      let new-value matrix:get dyad-scores who p - item who agent-individual-gains - item p agent-individual-gains
      matrix:set shapley-values who p new-value      
      set p p + 1
    ]
  ]
end

to form-coalitions
  let turtle-count count turtles
  ask turtles [
    let p 0
    let max-score 0
    let current-score 0
    let max-index -1
    while [p < turtle-count] [
      if p != who [
        if relationship-neighbor? turtle p [ ;; only form a coalition if a relationship is present
          set current-score matrix:get shapley-values who p
          if current-score > max-score [
            set max-score current-score
            set max-index p
          ]
        ]
      ]
      set p p + 1
    ]
    ;; TODO: insert minimum treshold for coalition formation
    if max-index != -1 [
      ifelse coalition-neighbor? turtle max-index [
        ask coalition who max-index [
          set thickness thickness + 0.005
        ] 
      ][
        create-coalition-with turtle max-index [
          set color 15
          set thickness 0.3
        ]
        set number-of-coalitions number-of-coalitions + 1
      ]
    
      ask turtle who [
        set number-of-coalitions number-of-coalitions + 1
      ]
      ask turtle max-index [
        set number-of-coalitions number-of-coalitions + 1
      ]
    ]
  ]
end

to view-coalitions
  ;;print matrix:pretty-print-text coalition-matrix
  let turtle-count count turtles
  set coalition-counter 0
  set coalition-list []
  let q 0
  let current-turtle 0
  
  while [q < turtle-count][
    set coalition-list lput -1 coalition-list ;; fill the list with -1s
    set q q + 1 ;; increment q
  ]
  ask turtles [
    set current-turtle who
    ifelse item who coalition-list > 0[
      ;; do nothing
    ][
      set coalition-counter coalition-counter + 1 ;; increment
      set coalition-list replace-item current-turtle coalition-list coalition-counter;; add coalition-counter to coalition-list to express processed agent
      set color coalition-counter * 10 + 5 ;; change color
      let p 0
      while [p < turtle-count][ ;; find all coalition members
        if matrix:get coalition-matrix current-turtle p > 0 [ ;; if connected to agent p
          ask turtle p [
            ;;type "connection found: ";
            ;;type current-turtle;
            ;;type "&";
            ;;print p;
            set coalition-list replace-item p coalition-list coalition-counter;; set coalition-list to value greater than 0, such that it will not be processed twice.
            set color coalition-counter * 10 + 5 ;; give all connected agents that same color
          ]
        ]
        set p p + 1
      ]
    ]
  ]
  print coalition-list
end

to compute-coalition-strengths
  let max-value max coalition-list ;; number of coalitions
  let s 0 
  set coalition-strengths [] ;; create empty list
  while [s <= max-value] [ ;; iterate over all coalitions
    set coalition-strengths fput 0 coalition-strengths ;; fill list with zeros
    set s s + 1
  ]
  let p 0
  while [p <= max-value] [ ;; iterate over all coalitions
    let q 0
    while [q < length coalition-list] [ ;; iterate over all agents
      if item q coalition-list = p [ ;;if agent q is in coalition p
        let old-value item p coalition-strengths ;; save old value
        let new-value old-value + [item q agent-scores] of turtle q ;; old value + added agent's value
        set coalition-strengths replace-item p coalition-strengths new-value ;; replace with new coalition strength
      ]
      set q q + 1
    ]
    set p p + 1
  ]
  type "coalition strengths: " print coalition-strengths
end

to compute-winning-coalition
  let max-value max coalition-strengths
  set winning-coalition position max-value coalition-strengths
  update-wins winning-coalition
  type "winning coalition: " print winning-coalition
end

to update-power
  let index winning-coalition
  let coalition-strength item index coalition-strengths ;; get the strength of the winning coalition
  let p 0
  while [p < length coalition-list] [ ;; go through the list of agents
    if item p coalition-list = winning-coalition [ ;; if the agent belongs to the winning coalition
      ask turtle p [
        let extra-power total-payoff / coalition-strength * item p agent-scores ;; TODO: this is my the agent's own perceived score, may be wrong!
        set power power + extra-power ;; update the agent's power
      ]
    ]
    set p p + 1
  ]
end

to compute-averages-per-winning-coalition
  let index winning-coalition
  let coalition-strength item index coalition-strengths ;; get the strength of the winning coalition
  let p 0
  let count-winners 0
  
  let cumulative-idea-value-per-winning-coalition 0 
  set average-idea-value-per-winning-coalition 0
  
  let cumulative-betweenness-per-winning-coalition 0 
  set average-betweenness-per-winning-coalition 0
  
  let cumulative-power-per-winning-coalition 0
  set average-power-per-winning-coalition 0
  
  while [p < length coalition-list] [ ;; go through the list of agents
    if item p coalition-list = winning-coalition [ ;; if the agent belongs to the winning coalition    
        set cumulative-betweenness-per-winning-coalition cumulative-betweenness-per-winning-coalition + item p CB ;;add betweenness centrality to total
    
        ask turtle p[
          set cumulative-power-per-winning-coalition cumulative-power-per-winning-coalition + power ;;add power to total
          set cumulative-idea-value-per-winning-coalition cumulative-idea-value-per-winning-coalition + idea-value ;;add idea value to total
        ]
        set count-winners count-winners + 1
    ]
    set p p + 1
  ]
  set average-idea-value-per-winning-coalition cumulative-idea-value-per-winning-coalition / count-winners ;; compute average
  set average-betweenness-per-winning-coalition cumulative-betweenness-per-winning-coalition / count-winners ;; compute average
  set average-power-per-winning-coalition cumulative-power-per-winning-coalition / count-winners ;; compute average
end

to compute-maximums-per-winning-coalition
  let index winning-coalition ;; get the number of the winning coalition
  let coalition-strength item index coalition-strengths ;; get the strength of the winning coalition
  let p 0
  let count-winners 0
  
  set max-idea-value-per-winning-coalition 0 
  
  set max-betweenness-per-winning-coalition 0 
  
  set max-power-per-winning-coalition 0
  
  while [p < length coalition-list] [ ;; go through the list of agents
    if item p coalition-list = winning-coalition [ ;; if the agent belongs to the winning coalition    
      if item p CB > max-betweenness-per-winning-coalition [ ;; if current betweenness greater than max
        set max-betweenness-per-winning-coalition item p CB ;;replace betweenness centrality in max-betweenness-per-winning-coalition
      ]
      ask turtle p[
        if power > max-power-per-winning-coalition [;; if power is greater than max
          set max-power-per-winning-coalition power ;; replace max-power-per-winning-coalition with power
        ]
        if idea-value > max-idea-value-per-winning-coalition [ ;;if idea-value is greater than max
          set max-idea-value-per-winning-coalition idea-value ;;replace max-idea-value-per-winning-coalition with current idea-value
        ]
      ]
      set count-winners count-winners + 1
    ]
    set p p + 1
  ]
end

to compute-average-power
  ask turtles [
    set power-average (power-average * (ticks + 1) + power) / (ticks + 2)
  ]
end

to compute-average-idea-value
  ask turtles [
    set idea-value-average (idea-value-average * (ticks + 1) + idea-value) / (ticks + 2)
  ]
end

to view-number-of-coalitions
  set total-number-of-coalitions coalition-counter 
end

to update-wins [coalition-number]
  let p 0
  set winning-coalition-size 0
  set chance-of-winning-coalition 0
  
  while [p < length coalition-list] [ ;; go through the list of agents
    let new-value 0
    let old-value 0
    if item p coalition-list = coalition-number [ ;; if the agent belongs to the winning coalition
      set winning-coalition-size winning-coalition-size + 1
      ask turtle p [
        set number-of-wins number-of-wins + 1
      ]
      set old-value item p wins-list
      set new-value old-value + 1
      set wins-list replace-item p wins-list new-value ;; add 1 to number of wins for agent p
    ]
    set p p + 1
  ]
  set chance-of-winning-coalition winning-coalition-size / number-of-turtles
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; environment functions
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

to find-path-lengths
  ;; reset the distance list
  ask turtles [
    set distance-from-other-turtles []
  ]

  let i 0
  let j 0
  let k 0
  let node1 one-of turtles
  let node2 one-of turtles
  let node-count count turtles
  ;; initialize the distance lists
  while [i < node-count] [
    set j 0
    while [j < node-count] [
      set node1 turtle i
      set node2 turtle j
      ;; zero from a node to itself
      ifelse i = j [
        ask node1 [
          set distance-from-other-turtles lput 0 distance-from-other-turtles
        ]
      ][
        ;; 1 from a node to it's neighbor
        ifelse [ relationship-neighbor? node1 ] of node2 [
          ask node1 [
            set distance-from-other-turtles lput 1 distance-from-other-turtles
          ]
        ]
        ;; infinite to everyone else
        [
          ask node1 [
            set distance-from-other-turtles lput infinity distance-from-other-turtles
          ]
        ]
      ]
      set j j + 1
    ]
    set i i + 1
  ]
  set i 0
  set j 0
  let dummy 0
  while [k < node-count] [
    set i 0
    while [i < node-count] [
      set j 0
      while [j < node-count] [
        ;; alternate path length through kth node
        set dummy ( (item k [distance-from-other-turtles] of turtle i) +
                    (item j [distance-from-other-turtles] of turtle k))
        ;; is the alternate path shorter?
        if dummy < (item j [distance-from-other-turtles] of turtle i) [
          ask turtle i [
            set distance-from-other-turtles replace-item j distance-from-other-turtles dummy
          ]
        ]
        set j j + 1
      ]
      set i i + 1
    ]
    set k k + 1
  ]
end

to find-coalition-paths
  ;; reset the distance list
  ask turtles [
    set coalition-path-length []
  ]

  let i 0
  let j 0
  let k 0
  let node1 one-of turtles
  let node2 one-of turtles
  let node-count count turtles
  ;; initialize the distance lists
  while [i < node-count] [
    set j 0
    while [j < node-count] [
      set node1 turtle i
      set node2 turtle j
      ;; zero from a node to itself
      ifelse i = j [
        ask node1 [
          set coalition-path-length lput 0 coalition-path-length
          matrix:set coalition-matrix i j 0
        ]
      ][
        ;; 1 from a node to it's neighbor
        ifelse [ coalition-neighbor? node1 ] of node2 [
          ask node1 [
            set coalition-path-length lput 1 coalition-path-length
            matrix:set coalition-matrix i j 1
          ]
        ]
        ;; infinite to everyone else
        [
          ask node1 [
            set coalition-path-length lput infinity coalition-path-length
            matrix:set coalition-matrix i j 0
          ]
        ]
      ]
      set j j + 1
    ]
    set i i + 1
  ]
  set i 0
  set j 0
  let dummy 0
  while [k < node-count] [
    set i 0
    while [i < node-count] [
      set j 0
      while [j < node-count] [
        ;; alternate path length through kth node
        set dummy ( (item k [coalition-path-length] of turtle i) +
                    (item j [coalition-path-length] of turtle k))
        ;; is the alternate path shorter?
        if dummy < (item j [coalition-path-length] of turtle i) [
          ask turtle i [
            matrix:set coalition-matrix i j dummy
            set coalition-path-length replace-item j coalition-path-length dummy
          ]
        ]
        set j j + 1
      ]
      set i i + 1
    ]
    set k k + 1
  ]
  
  set i 0
  while [i < node-count] [
    set j 0
    while [j < node-count] [
        ask turtle i [
          if item j coalition-path-length = infinity [ ;; if path length is infinity
            set coalition-path-length replace-item j coalition-path-length -1 ;; then set to -1
          ]
        ]
      set j j + 1
    ]
    set i i + 1
  ]      
end

to calculate-betweenness-centrality
  set CB []
  let turtle-count count turtles
  let s 0
  let v2 0
  while [v2 < turtle-count] [
    set CB lput 0 CB ;; initialize the betweenness centralities to zero
    set v2 v2 + 1
  ]
  while [s < turtle-count] [ 
    let S-list [] ;; empty stack (LIFO)
    ;; make empty list of predecessors for every w in V (FIFO)
    let P-list matrix:make-constant turtle-count turtle-count 0 
    let sigma [] ;; empty sigma
    let depth [] ;; empty depth
    let Q-list [] ;; emtpy queue Q (FIFO queue)
    set Q-list fput s Q-list ;; enqueue s in Q 
      
    let p 0 
    while [p < turtle-count] [ ;;initialize sigma containing zeros
      set sigma fput 0 sigma  
      set p p + 1
    ]
    set sigma replace-item s sigma 1
    ;; except s, which is one
    
    let q 0 
    while [q < turtle-count] [ ;;initialize depth containing minus ones
      set depth fput -1 depth  
      set q q + 1
    ]
    set depth replace-item s depth 0
    ;; except s, which is one
    
    while [empty? Q-list = false] [
      let v last Q-list ;; get v (last item) from Q
      set Q-list but-last Q-list ;; remove v (last item) from Q
      set S-list fput v S-list ;; push v to S
      
      let w 0
      while [w < turtle-count] [
        if v != w [
          let node1 one-of turtles
          let node2 one-of turtles
          set node1 turtle w
          set node2 turtle v
          if [relationship-neighbor? node1] of node2 [ ;; if w and v are neighbors
            if item w depth < 0 [ ;; if w found for the first time
              set Q-list fput w Q-list ;; enqueue w in Q
              let v-depth item v depth ;; retrieve d[v]
              let w-depth v-depth + 1 ;; d[w] is d[v] plus one
              set depth replace-item w depth w-depth ;; replace depth of d[w]
            ]
            if item w depth = item v depth + 1 [ ;; shortest path to w via v?
              let sigma-w-plus-v item w sigma + item v sigma ;; add sigma w and sigma v
              set sigma replace-item w sigma sigma-w-plus-v ;; assign value to sigma[w]
              matrix:set P-list w v 1 ;; add v to predecessor list of w
            ]
          ]
        ]
        set w w + 1
      ]
    ]
      
    let v 0 
    let delta []
    while [v < turtle-count] [
      set delta fput 0 delta ;; set delta[v] to 0 for every v in V
      set v v + 1
    ]
    ;; S returns vertices in order of non-increasing distance from s
    while [empty? S-list = false] [
      let w first S-list ;; pop w from the top of S  
      set S-list but-first S-list;; remove w from the top of S
      let v1 0
      while [v1 < turtle-count] [
        if matrix:get P-list w v1 = 1 [ 
          let new-delta-v1 item v1 delta + item v1 sigma / item w sigma * (1 + item w delta)
          set delta replace-item v1 delta new-delta-v1 ;; for every v in P, calculate sigma[v] = sigma[v] + sigma[v]/sigma[w] * (1+sigma[w]) 
          if w != s [
            let CB-new-for-w item w CB + item w delta
            set CB replace-item w CB CB-new-for-w
          ]
        ]
        set v1 v1 + 1
      ]
    ]
    set s s + 1
  ]
  ;; type "Betweenness Centralities:" print CB
end

to calculate-power
  ask turtles [
    set old-power power
    set power item who CB * w1 + idea-value * w2 + age * w4 + old-power
  ]  
end

to set-weights
  set w1 0.45 
  set w2 0.45
  set w3 0.67
  set w4 0.1
  set w5 1
  set w6 1
  set w7 1
  set w8 0.25
  set w9 0.25
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; monitoring functions
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

to do-plots
  set-current-plot "Totals"
  ;;set-current-plot-pen "turtles"
  ;;plot count turtles
  set-current-plot-pen "coalitions"
  plot total-number-of-coalitions
  ;;set-current-plot-pen "relationships"
  ;;plot count relationships
  
  set-current-plot "turtle wins"
  
  
  ;;plot-pen-reset
  let turtle-count count turtles
  ;;set-histogram-num-bars turtle-count
  ;;histogram [number-of-wins] of turtles
  
  let w 0
  set-current-plot-pen "avg power"
  plot-pen-reset
  set-current-plot-pen "CB"
  plot-pen-reset
  while [w < turtle-count] [
    ask turtle w [
      set-current-plot-pen "wins"
      set-plot-pen-mode 1
      plotxy w number-of-wins
      set-current-plot-pen "avg power"
      set-plot-pen-mode 1
      plotxy w power-average
      set-current-plot-pen "CB"
      set-plot-pen-mode 1
      plotxy w item w CB
    ]
    set w w + 1;
  ]
  
  let v 0
  set-current-plot "plot 1"
  ;;plot betweenness x no. wins
  set-current-plot-pen "CB x wins"
  ;; plot-pen-reset
  while [v < turtle-count] [
    ask turtle v [
      set-plot-pen-mode 2
      plotxy number-of-wins item v CB
    ]
    set v v + 1
  ]
  
  ;; plot power x no. wins
  set-current-plot-pen "power x wins"
  ;; uncomment to have avg power at time point 100 vs no. wins 
  ;;plot-pen-reset
  let p 0
  while [p < turtle-count] [
    ask turtle p [
      set-plot-pen-mode 2
      plotxy number-of-wins power-average
    ]
    set p p + 1
  ]
  
  ;; plot power x no. wins
  set-current-plot-pen "idea value x wins"
  ;; uncomment to have avg power at time point 100 vs no. wins 
  ;;plot-pen-reset
  let c 0
  while [c < turtle-count] [
    ask turtle c [
      set-plot-pen-mode 2
      plotxy number-of-wins idea-value-average
    ]
    set c c + 1
  ]
  
  set-current-plot "average power per winning coalition"
  ;;set-current-plot-pen "turtles"
  ;;plot count turtles
  set-current-plot-pen "avg-power"
  plot average-power-per-winning-coalition
  set-current-plot-pen "avg-Cb"
  plot average-betweenness-per-winning-coalition
  set-current-plot-pen "avg-idea-value"
  plot average-idea-value-per-winning-coalition
end

to get-skills-from-agent [turtlenumber]
  type "skills turtle " type turtlenumber type " owns: " print [skills] of turtle turtlenumber
end
@#$#@#$#@
GRAPHICS-WINDOW
288
13
727
473
16
16
13.0
1
10
1
1
1
0
1
1
1
-16
16
-16
16
0
0
1
ticks

BUTTON
46
10
112
43
NIL
setup
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL

BUTTON
119
10
182
43
NIL
go
T
1
T
OBSERVER
NIL
NIL
NIL
NIL

MONITOR
6
234
99
279
turtles
count turtles
17
1
11

PLOT
747
222
1019
372
Totals
time
frequency
0.0
100.0
0.0
10.0
true
true
PENS
"coalitions" 1.0 0 -2674135 true

SLIDER
6
117
178
150
number-of-turtles
number-of-turtles
0
100
100
1
1
NIL
HORIZONTAL

CHOOSER
6
187
144
232
network-type
network-type
"random2" "random" "regular lattice" "small world"
0

MONITOR
101
234
174
279
coalitions
total-number-of-coalitions
17
1
11

SWITCH
6
152
177
185
hide-relationships
hide-relationships
0
1
-1000

PLOT
7
282
282
432
turtle wins
turtles
score
0.0
10.0
0.0
100.0
true
true
PENS
"wins" 1.0 0 -2674135 true
"avg power" 1.0 0 -16777216 true
"CB" 1.0 0 -10899396 true

PLOT
749
16
1042
201
plot 1
wins
score
0.0
100.0
0.0
100.0
true
true
PENS
"CB x wins" 1.0 0 -16777216 true
"power x wins" 1.0 2 -2674135 true
"idea value x wins" 1.0 0 -10899396 true

MONITOR
176
235
287
280
network density
count relationships / (count turtles * count turtles / 2)
17
1
11

SLIDER
6
82
178
115
network-density
network-density
0
1
0.05
0.01
1
NIL
HORIZONTAL

PLOT
750
383
1042
533
average power per winning coalition
time
score
0.0
100.0
0.0
100.0
true
true
PENS
"avg-power" 1.0 0 -2674135 true
"avg-Cb" 1.0 0 -16777216 true
"avg-idea-value" 1.0 0 -10899396 true

MONITOR
18
451
245
496
max-power-per-winning-coalition
max-power-per-winning-coalition
17
1
11

@#$#@#$#@
WHAT IS IT?
-----------
This section could give a general understanding of what the model is trying to show or explain.


HOW IT WORKS
------------
This section could explain what rules the agents use to create the overall behavior of the model.


HOW TO USE IT
-------------
This section could explain how to use the model, including a description of each of the items in the interface tab.


THINGS TO NOTICE
----------------
This section could give some ideas of things for the user to notice while running the model.


THINGS TO TRY
-------------
This section could give some ideas of things for the user to try to do (move sliders, switches, etc.) with the model.


EXTENDING THE MODEL
-------------------
This section could give some ideas of things to add or change in the procedures tab to make the model more complicated, detailed, accurate, etc.


NETLOGO FEATURES
----------------
This section could point out any especially interesting or unusual features of NetLogo that the model makes use of, particularly in the Procedures tab.  It might also point out places where workarounds were needed because of missing features.


RELATED MODELS
--------------
This section could give the names of models in the NetLogo Models Library or elsewhere which are of related interest.


CREDITS AND REFERENCES
----------------------
This section could contain a reference to the model's URL on the web if it has one, as well as any other necessary credits or references.
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 4.1.3
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
<experiments>
  <experiment name="experiment3" repetitions="10" runMetricsEveryStep="true">
    <setup>setup</setup>
    <go>go</go>
    <metric>average-power-per-winning-coalition</metric>
    <metric>average-betweenness-per-winning-coalition</metric>
    <metric>average-idea-value-per-winning-coalition</metric>
    <metric>max-power-per-winning-coalition</metric>
    <metric>max-betweenness-per-winning-coalition</metric>
    <metric>max-idea-value-per-winning-coalition</metric>
    <metric>total-number-of-coalitions</metric>
    <metric>winning-coalition-size</metric>
    <metric>chance-of-winning-coalition</metric>
    <steppedValueSet variable="network-density" first="0.01" step="0.01" last="0.05"/>
    <enumeratedValueSet variable="hide-relationships">
      <value value="true"/>
    </enumeratedValueSet>
    <steppedValueSet variable="number-of-turtles" first="10" step="10" last="100"/>
    <enumeratedValueSet variable="network-type">
      <value value="&quot;random2&quot;"/>
    </enumeratedValueSet>
  </experiment>
  <experiment name="experiment2" repetitions="10" runMetricsEveryStep="true">
    <setup>setup</setup>
    <go>go</go>
    <metric>average-power-per-winning-coalition</metric>
    <metric>average-betweenness-per-winning-coalition</metric>
    <metric>average-idea-value-per-winning-coalition</metric>
    <metric>max-power-per-winning-coalition</metric>
    <metric>max-betweenness-per-winning-coalition</metric>
    <metric>max-idea-value-per-winning-coalition</metric>
    <metric>total-number-of-coalitions</metric>
    <steppedValueSet variable="network-density" first="0.01" step="0.01" last="0.05"/>
    <enumeratedValueSet variable="hide-relationships">
      <value value="true"/>
    </enumeratedValueSet>
    <steppedValueSet variable="number-of-turtles" first="2" step="1" last="30"/>
    <enumeratedValueSet variable="network-type">
      <value value="&quot;random2&quot;"/>
    </enumeratedValueSet>
  </experiment>
</experiments>
@#$#@#$#@
@#$#@#$#@
default
0.0
-0.2 0 0.0 1.0
0.0 1 1.0 0.0
0.2 0 0.0 1.0
link direction
true
0
Line -7500403 true 150 150 90 180
Line -7500403 true 150 150 210 180

@#$#@#$#@
0
@#$#@#$#@
